<?php

namespace Bixo\Recomentations\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Recomentations\Http\Requests\RecomentationRequest;
use Bixo\Recomentations\Interfaces\RecomentationRepositoryInterface;
use Bixo\Recomentations\Models\Recomentation;

/**
 * Resource controller class for recomentation.
 */
class RecomentationResourceController extends BaseController
{

    /**
     * Initialize recomentation resource controller.
     *
     * @param type RecomentationRepositoryInterface $recomentation
     *
     * @return null
     */
    public function __construct(RecomentationRepositoryInterface $recomentation)
    {
        parent::__construct();
        $this->repository = $recomentation;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Recomentations\Repositories\Criteria\RecomentationResourceCriteria::class);
    }

    /**
     * Display a list of recomentation.
     *
     * @return Response
     */
    public function index(RecomentationRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Recomentations\Repositories\Presenter\RecomentationPresenter::class)
                ->$function();
        }

        $recomentations = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('recomentations::recomentation.names'))
            ->view('recomentations::recomentation.index', true)
            ->data(compact('recomentations', 'view'))
            ->output();
    }

    /**
     * Display recomentation.
     *
     * @param Request $request
     * @param Model   $recomentation
     *
     * @return Response
     */
    public function show(RecomentationRequest $request, Recomentation $recomentation)
    {

        if ($recomentation->exists) {
            $view = 'recomentations::recomentation.show';
        } else {
            $view = 'recomentations::recomentation.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('recomentations::recomentation.name'))
            ->data(compact('recomentation'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new recomentation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RecomentationRequest $request)
    {

        $recomentation = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('recomentations::recomentation.name')) 
            ->view('recomentations::recomentation.create', true) 
            ->data(compact('recomentation'))
            ->output();
    }

    /**
     * Create new recomentation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RecomentationRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $recomentation                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('recomentations::recomentation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('recomentations/recomentation/' . $recomentation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/recomentations/recomentation'))
                ->redirect();
        }

    }

    /**
     * Show recomentation for editing.
     *
     * @param Request $request
     * @param Model   $recomentation
     *
     * @return Response
     */
    public function edit(RecomentationRequest $request, Recomentation $recomentation)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('recomentations::recomentation.name'))
            ->view('recomentations::recomentation.edit', true)
            ->data(compact('recomentation'))
            ->output();
    }

    /**
     * Update the recomentation.
     *
     * @param Request $request
     * @param Model   $recomentation
     *
     * @return Response
     */
    public function update(RecomentationRequest $request, Recomentation $recomentation)
    {
        try {
            $attributes = $request->all();

            $recomentation->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('recomentations::recomentation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('recomentations/recomentation/' . $recomentation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('recomentations/recomentation/' . $recomentation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the recomentation.
     *
     * @param Model   $recomentation
     *
     * @return Response
     */
    public function destroy(RecomentationRequest $request, Recomentation $recomentation)
    {
        try {

            $recomentation->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('recomentations::recomentation.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('recomentations/recomentation/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('recomentations/recomentation/' . $recomentation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple recomentation.
     *
     * @param Model   $recomentation
     *
     * @return Response
     */
    public function delete(RecomentationRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('recomentations::recomentation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('recomentations/recomentation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/recomentations/recomentation'))
                ->redirect();
        }

    }

    /**
     * Restore deleted recomentations.
     *
     * @param Model   $recomentation
     *
     * @return Response
     */
    public function restore(RecomentationRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('recomentations::recomentation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/recomentations/recomentation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/recomentations/recomentation/'))
                ->redirect();
        }

    }

}
